//Under MIT License
//Do not edit & redistibute without permission
//Designed for Paint.NET
//2006 Robert Nix, aliased Illnab1024
//
//Effect achieved is a 256 colors effect. It does not dither for aesthetics.
//
//Thank you, Rick, Tom, MSFT, WSU, and all others who made PdN possible.

using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace PaintDotNet.Effects
{
    [EffectTypeHint(EffectTypeHint.Fast)]
    public class CutoutEffect
        : Effect
    {
        public static string StaticName
        {
            get
            {
                return "Cutout";
            }
        }

        public CutoutEffect()
            : base(StaticName,
                   PdnResources.GetImage("Icons.PixelateEffect.png"),
                   true)
        {
        }


        public override EffectConfigDialog CreateConfigDialog()
        {
            AmountEffectConfigDialog config = new AmountEffectConfigDialog();

            config.Effect = this;
            config.Text = "Interval Amount";
            config.SliderMinimum = 1;
            config.SliderMaximum = 255;
            config.SliderLabel = "Choose an Amount";
            config.SliderUnitsName = "Interval Colors";
            config.Icon = PdnResources.GetIconFromImage("Icons.PixelateEffect.png");
            return config;
        }

        public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
        {
            Surface dst = dstArgs.Surface;
            Surface src = srcArgs.Surface;
            AmountEffectConfigToken config = (AmountEffectConfigToken)parameters;
            int interval = (config.Amount);
            int R;
            int B;
            int G;
            int A;
            float divR;
            float divB;
            float divG;
            float divA;
            for (int ri = startIndex; ri < startIndex + length; ++ri)
            {
                Rectangle rect = rois[ri];

                for (int y = rect.Top; y < rect.Bottom; y++)
                {
                    for (int x = rect.Left; x < rect.Right; x++)
                    {
                        ColorBgra source = src[x, y];
                        divR = source.R / interval;
                        divB = source.B / interval;
                        divG = source.G / interval;
                        divA = source.A / interval;
                        R = (int)System.Math.Ceiling(divR);
                        B = (int)System.Math.Ceiling(divB);
                        G = (int)System.Math.Ceiling(divG);
                        A = (int)System.Math.Ceiling(divA);
                        R *= interval;
                        B *= interval;
                        G *= interval;
                        A = 255; //A * interval;
                        ColorBgra fin = dst[x, y];
                        fin.R = (byte)(R);
                        fin.B = (byte)(B);
                        fin.G = (byte)(G);
                        fin.A = (byte)(A);
                        dst[x, y] = fin;
                    }
                }
            }
        }
    }
}